(define-syntax-rule (values&forms . forms)
  "\
Return a list of pairs, one per argument.  Each pair contains the value of
the argument as the car and the unevaluated expression as the cdr.
(values&forms (* 3 5) (string-append \"foo\" \"bar\"))
=> ((15 . (* 3 5)) (\"foobar\" . (string-append \"foo\" \"bar\")))"
  (map cons (list . forms) 'forms))

(define-syntax qw
  (syntax-rules ()
    "\
Return a list of strings, analogous to Perl's qw().  Each argument can be a
string, which is included as-is; a symbol, whose name is used; or an
expression, which is evaluated.  If the expression yields a list value, the
elements of that list are interpolated directly into the returned list.
(qw \"foo\" bar (list \"baz\" \"quux\"))
=> (\"foo\" \"bar\" \"baz\" \"quux\")"
    ((_)
     '())
    ((_ form forms ...)
     (append (cond
              ((string? 'form) (list form))
              ((symbol? 'form) (list (symbol->string 'form)))
              (else
               (let ((x form))
                 (if (list? x)
                   x
                   (list x)))))
             (qw forms ...)))))

(define-syntax scope-1
  (syntax-rules (defn)
    ((_ ((defn var value)))
     value)
    ((_ ((defn var value) . forms))
     (let ((var value))
       (scope-1 forms)))
    ((_ (form))
     form)
    ((_ (form . forms))
     (begin
       form
       (scope-1 forms)))))
(define-syntax-rule (scope . forms)
  "\
Like let*, but allows definitions like (defn var value) to be interleaved
with other expressions.
(scope
  (defn x 3)
  (simple-format #t \"~S~%\" (* x x))
  (defn y (+ x 2))
  (* x y))
=> 15
"
  (scope-1 forms))

(define-syntax-rule (defproc-by definer (name . args) . forms)
  "\
Define a procedure, but also return the procedure."
  (let ()
    (definer (name . args) . forms)
    name))

(define-syntax-rule (defproc (name . args) . forms)
  "\
Define a procedure, like (define (...) ...), but also return the procedure."
  (defproc-by define (name . args) . forms))

(define-syntax-rule (defproc* prototype . forms)
  "\
Define a procedure, like (define* (...) ...), but also return the procedure."
  (defproc-by define* (name . args) . forms))
